// Samoposluga.java - Samoposluga.

package samoposluga;
import usluge.*;
import java.awt.Color;

public class Samoposluga {

  private static int ukId = 0;
  private int id = ++ukId;
  private Ulaz[] ulazi;
  private Kasa[] kase;
  private Polje polje;
  private volatile int brojKupaca = 0;
  private boolean radi = false;

  public Samoposluga (int brUlaza, int brKasa, 
               int maxUlaz, int maxKupovina, int maxNaplata,
               Polje p, Polje[] pUlazi,
               Polje[] pKase, Polje[] pRedovi) {
    if ((polje = p) != null) {
      polje.komponenta().setForeground (Color.red);
      polje.pisi ("Kupaca: 0");
    }
    ulazi = new Ulaz [brUlaza];
    for (int i=0; i<brUlaza; i++) {
      ulazi[i] = new Ulaz (this, maxUlaz, maxKupovina,
                           pUlazi!=null?pUlazi[i]:null);
    }
    kase  = new Kasa [brKasa];
    for (int i=0; i<brKasa; i++)
      kase[i] = new Kasa (this, maxNaplata,
                          pKase!=null?pKase[i]:null, 
                          pRedovi!=null?pRedovi[i]:null);
  }

  public void otvori () {
    if (polje != null) polje.komponenta().setForeground (Color.black);
    radi = true;
    for (int i=0; i<ulazi.length; ulazi[i++].otvori());
  }

  public void zatvori () {
    if (polje != null) polje.komponenta().setForeground (Color.red);
    radi = false;
    for (int i=0; i<ulazi.length; ulazi[i++].zatvori());
    synchronized (this) {
      while (brojKupaca > 0) 
        try { wait (); } catch (InterruptedException e) {}
    }
  }

  synchronized void usaoKupac   () {
    brojKupaca++; 
    if (polje != null) polje.pisi ("Kupaca: " + brojKupaca);
  }

  synchronized void izasaoKupac () { 
    if (--brojKupaca == 0 && !radi) notify ();
    if (polje != null) polje.pisi ("Kupaca: " + brojKupaca);
  }

  public int id () { return id; }

  Kasa[] dohvatiKase () { return kase; }

  public void unisti () {
    if (radi) zatvori ();
    for (int i=0; i<ulazi.length; ulazi[i++].zavrsi());
    for (int i=0; i<kase .length; kase [i++].zavrsi());
  }
  
} // class Samoposluga
